--
--  Platformer Tutorial
--

local loader = require "AdvTiledLoader/Loader"
-- set the path to the Tiled map files
loader.path = "maps/"

local HC = require "HardonCollider"

local camera = require("hump.camera")

local heart
local hero
local boss
local monsters = {}
local collider
local allSolidTiles
local cam
local projectiles = {}
local map
local current_popup = {
	img = nil,
	lifetime = 0,
	callback = nil,
	y = 20
}
local popups = {}
local pause = false

local sounds = {}

local anims = {
	idle1 = 1,
	idle2 = 2,
	walk1 = 3,
	walk2 = 4,
	walk3 = 5,
	walk4 = 6,
	jump = 7,
	action1 = 8,
	action2 = 9
}

local fishes = {
	none = 1,
	jumpfish = 2,
	glidefish = 3,
	bubblefish = 4,
	swordfish = 5
}

local joystick = love.joystick.getJoysticks()[1]

function love.load()

	cam = camera.new()

	-- load the level and bind to variable map
	map = loader.load("bosstest1 - Copy.tmx")
	--object_offset = 53
	--map = loader.load("iwannabethefish.tmx")
	object_offset = -1
	map:autoDrawRange(0, 0, 1, 0)
	--map.drawObjects = false

	-- load HardonCollider, set callback to on_collide and size of 100
	collider = HC(100, on_collide)

	-- find all the tiles that we can collide with
	allSolidTiles = findSolidTiles(map)

	-- fix pickups
	if map("fisk") then
		for i, obj in pairs(map("fisk").objects) do
			obj.gid = obj.properties.image

			local ctile = collider:addRectangle(obj.x, obj.y, obj.width, obj.height)
			ctile.type = "pickup"
			ctile.obj = obj
			obj.ctile = ctile
			collider:addToGroup("pickups", ctile)
			collider:setPassive(ctile)
			if obj.properties.pond then
				collider:setGhost(ctile)
				obj.visible = false
			end
		end
	end
	-- fix signs
	if map("signs") then
		for i, obj in pairs(map("signs").objects) do
			obj.gid = obj.properties.image

			--print(obj.name .. "," .. obj.x .. "," .. obj.y .. "," .. obj.width .. "," .. obj.height)
			local ctile = collider:addRectangle(obj.x, obj.y, obj.width, obj.height)
			ctile.type = "sign"
			ctile.obj = obj
			collider:addToGroup("signs", ctile)
			collider:setPassive(ctile)
		end
	end
	-- fix monsters
	if map("monster") then
		for i, obj in pairs(map("monster").objects) do
			obj.gid = obj.properties.image

			local ctile = collider:addRectangle(obj.x, obj.y, obj.width, obj.height)
			ctile.type = "monster"
			ctile.obj = obj
			collider:addToGroup("monsters", ctile)
			collider:setPassive(ctile)
			table.insert(monsters, ctile)

			ctile.respawn = 0
			ctile.stuntime = 0
			ctile.anim_state = 1

			ctile.x_vel = 70
		end
	end
	-- fix targets
	if map("targets") then
		for i, obj in pairs(map("targets").objects) do
			obj.gid = obj.properties.image

			local ctile = collider:addRectangle(obj.x, obj.y, obj.width, obj.height)
			ctile.type = "target"
			ctile.obj = obj
			collider:addToGroup("targets", ctile)
			collider:setPassive(ctile)
		end
	end
	-- fix walls
	if map("walls") then
		for i, obj in pairs(map("walls").objects) do
			obj.gid = obj.properties.image

			local ctile = collider:addRectangle(obj.x, obj.y, obj.width, obj.height)
			ctile.type = "ground"
			obj.ctile = ctile
			collider:addToGroup("walls", ctile)
			collider:setPassive(ctile)
			obj.ghosted = false
			if obj.properties.start_ghost then
				obj.ghosted = true
				collider:setGhost(ctile)
				obj.visible = false
			end
			obj.lifetime = 0
		end
	end

	-- set up the hero object, set him to position 32, 32
	x, y = 96, 132
	if map("start") then
		local startpos = map("start").objects[1]
		startpos.visible = false
		x, y = startpos.x, startpos.y
	end
	setupHero(x, y)

	sounds.music = love.audio.newSource("sound/bosselicious.ogg", "stream")
	sounds.fanfare = love.audio.newSource("sound/fanfare.ogg", "static")
	sounds.fanfare2 = love.audio.newSource("sound/fanfare2.ogg", "static")
	sounds.death = love.audio.newSource("sound/death.ogg", "static")
	sounds.shot = love.audio.newSource("sound/laser shot.ogg", "static")
	sounds.whistle = love.audio.newSource("sound/whistle.ogg", "static")
	sounds.stabstop = love.audio.newSource("sound/dark tone.ogg", "static")
	sounds.attack = love.audio.newSource("sound/high tone.ogg", "static")
	sounds.hit = love.audio.newSource("sound/sesam.ogg", "static")
	sounds.music:play()

	popups[fishes.jumpfish] = love.graphics.newImage("img/salmon.png")
	popups[fishes.glidefish] = love.graphics.newImage("img/manta_ray.png")
	popups[fishes.bubblefish] = love.graphics.newImage("img/puffer_fish.png")
	popups[fishes.swordfish] = love.graphics.newImage("img/swordfish.png")
	popups.start = love.graphics.newImage("img/start.png")
	popups.gameover = love.graphics.newImage("img/game_over.png")
	popups.deliver = love.graphics.newImage("img/msg_deliver.png")
	popups.success = love.graphics.newImage("img/end.png")
	popups.death = love.graphics.newImage("img/death.png")

	heart = love.graphics.newImage("img/heart.png")

	love.graphics.setBackgroundColor(164, 188, 242)

	projectiles.img = love.graphics.newImage("img/bubble.png")
	projectiles.quads = {}
	for y_index = 1,2 do
		local bubble_anims = {}
		for anim_index = 1,5 do
			bubble_anims[anim_index] = love.graphics.newQuad((anim_index-1)*14, (y_index-1)*14, 14, 14, projectiles.img:getDimensions())
		end
		projectiles.quads[y_index] = bubble_anims
	end

	projectiles.img2 = love.graphics.newImage("img/sword_attack.png")
	projectiles.quads2 = {}
	for x_index = 1,2 do
		local sword_anims = {}
		for anim_index = 1,4 do
			sword_anims[anim_index] = love.graphics.newQuad((x_index-1)*16, (anim_index-1)*8, 16, 8, projectiles.img2:getDimensions())
		end
		projectiles.quads2[x_index] = sword_anims
	end


	hero.in_control = false
	current_popup.callback = function()
		hero.in_control = true
	end
	current_popup.y = 150
	current_popup.lifetime = 5
	current_popup.img = popups.start
end

function change_fish(fish)
	if hero.action then return end

	if hero.fish_inventory[fish] == true then
		hero.current_fish = fish
	end
end

local fish_actions = {
	pressed = {},
	held = {},
	released = {}
}

fish_actions.held[fishes.none] = function(dt)
	if not hero.jumping and hero.x_vel == 0 then
		sounds.whistle:play()
		hero.action_state = animate_state(hero.action_state, 2*dt, anims.action1, anims.action2)
	else
		sounds.whistle:stop()
	end
end

fish_actions.released[fishes.none] = function(dt)
	sounds.whistle:stop()
end

fish_actions.pressed[fishes.jumpfish] = function()
	if not hero.jumping and not hero.action then
		hero.action = true
	end
end

fish_actions.held[fishes.jumpfish] = function(dt)
	if hero.action then
		hero.x_vel = 0
		hero.action_state = hero.action_state + 5*dt
		if hero.action_state >= anims.action2-anims.action1+1-0.01 then
			hero.action_state = 0
			hero.action = false
			hero.current_anim = anims.jump
		else
			hero.current_anim = math.floor(anims.action1 + hero.action_state)
			if hero.current_anim == anims.action2 then
				jump(450)
			end
		end
	end
end

fish_actions.pressed[fishes.glidefish] = function()
	if hero.jumping then
		hero.current_anim = anims.action1
		hero.action = true
	end
end

fish_actions.released[fishes.glidefish] = function()
	if hero.action then
		hero.current_anim = anims.jump
		hero.action = false
	end
end

fish_actions.held[fishes.glidefish] = function(dt)
	if hero.action then
		if not hero.jumping then
			hero.action = false
			return
		end

		if hero.y_vel < -50 then hero.y_vel = -50 end
		hero.action_state = animate_state(hero.action_state, 2*dt, anims.action1, anims.action2)
	end
end

fish_actions.pressed[fishes.bubblefish] = function(dt)
	if not hero.action then
		hero.action = true
		sounds.shot:play()

		local attack_x, attack_y = hero:center()
		if hero.facing_left then
			attack_x = attack_x - 32
		else
			attack_x = attack_x + 32
		end

		local projectile = collider:addRectangle(attack_x, attack_y, 10, 10)
		projectile.type = "projectile"
		projectile.lifetime = 2.5
		projectile.img = projectiles.img
		projectile.quads = projectiles.quads
		projectile.x_vel = 140
		projectile.y_vel = -5
		if hero.facing_left then projectile.x_vel = -projectile.x_vel end
		table.insert(projectiles, projectile)
	end
end

fish_actions.held[fishes.bubblefish] = function(dt)
	if hero.action then
		hero.action_state = hero.action_state + 5*dt
		if hero.action_state >= anims.action2-anims.action1+1-0.01 then
			hero.action_state = 0
			hero.action = false
			hero.current_anim = anims.idle1
		else
			hero.current_anim = math.floor(anims.action1 + hero.action_state)
		end
	end
end

fish_actions.pressed[fishes.swordfish] = function(dt)
	if hero.action then return end

	print("ha!")
	hero.action = true
	hero.action_state = 0

	local attack_x, attack_y = hero:center()
	if hero.facing_left then
		attack_x = attack_x - 40
	else
		attack_x = attack_x + 40
	end
	--attack_x = attack_x + (hero.facing_left and -32) or 32
	local targets = collider:shapesAt(attack_x, attack_y)
	for i, shape in ipairs(targets) do
		if shape.type == "monster" then
			if shape.obj.visible then
				print("hit!")
				shape.obj.visible = false
				collider:setGhost(shape)
				shape.respawn = shape.obj.properties.respawn or 5

				if shape.obj.name == "boss1" then
					local calla = function()
						current_popup.y = 0
						current_popup.lifetime = 0.1
						current_popup.img = popups.gameover
						hero.in_control = true
						pause = true
						local callb
						callb = function()
							if current_popup.y > -800 then
								current_popup.img = popups.gameover
								current_popup.y = current_popup.y - 5
								current_popup.callback = callb
								current_popup.lifetime = 0.1
							else
								pause = false
							end
						end
						current_popup.callback = callb
					end
					current_popup.lifetime = 2
					hero.in_control = false
					hero.x_vel = 0
					current_popup.callback = calla
				end
			end
		elseif shape.type == "ground" and hero.jumping then
			local vel = -500
			if hero.facing_left then vel = -vel end
			hero.x_vel = vel
			hero.y_vel = 150
			hero.stabfly = true
			hero.in_control = false
			hero.action = false
		end
	end

	local projectile = collider:addRectangle(attack_x, attack_y, 10, 10)
	collider.setPassive(projectile)
	projectile.type = "projectile2"
	projectile.lifetime = 0.4
	projectile.img = projectiles.img2
	projectile.quads = projectiles.quads2
	projectile.x_vel = 10
	projectile.y_vel = 0
	if hero.facing_left then projectile.x_vel = -projectile.x_vel end
	table.insert(projectiles, projectile)

	sounds.attack:play()
end

fish_actions.held[fishes.swordfish] = function(dt)
	if hero.action then
		if not hero.stabfly then hero.x_vel = 0 end
		hero.action_state = hero.action_state + 7*dt
		if hero.action_state >= anims.action2-anims.action1+1-0.01 then
			hero.action_state = 0
			hero.action = false
			hero.current_anim = anims.idle1
		else
			hero.current_anim = math.floor(anims.action1 + hero.action_state)
		end
	end
end

function love.keypressed(k)
	if k == 'escape' then
		love.event.push('quit')
	end

	if k == "return" then
		current_popup.lifetime = 0.1
	end

	if not hero.in_control then
		return
	end

	if k == '1' then
		change_fish(fishes.none)
		local fun = fish_actions.pressed[hero.current_fish]
		if fun then fun() end
	elseif k == '2' then
		change_fish(fishes.jumpfish)
		local fun = fish_actions.pressed[hero.current_fish]
		if fun then fun() end
	elseif k == '3' then
		change_fish(fishes.glidefish)
		local fun = fish_actions.pressed[hero.current_fish]
		if fun then fun() end
	elseif k == '4' then
		change_fish(fishes.bubblefish)
		local fun = fish_actions.pressed[hero.current_fish]
		if fun then fun() end
	elseif k == '5' then
		change_fish(fishes.swordfish)
		local fun = fish_actions.pressed[hero.current_fish]
		if fun then fun() end
	end

	if k == 'z' then
		local fun = fish_actions.pressed[hero.current_fish]
		if fun then fun() end
	end
end

function love.keyreleased(k)
	if hero.in_control and (k == 'z' or k == '3') then
		local fun = fish_actions.released[hero.current_fish]
		if fun then fun() end
	end
end

function math.sign(x)
	local ax = math.abs(x)
	return (ax < 0.001 and 1) or (x / ax)
end

function closer(x, change)
	local r = math.max(0, math.abs(x) - change)
	return r * math.sign(x)
end

local dt_left = 0

function love.update(dt)
	if not pause then
		dt_left = dt_left + dt

		-- do all the input and movement
		while dt_left > 1/60 do
			dt_left = dt_left - 1/60
			local dt = 1/60

			if hero.in_control then
				handleInput(dt)
			end
			updateHero(dt)

			-- update monsters
			for i, monster in ipairs(monsters) do
				if monster.obj.visible then
					if monster.stuntime > 0 then
						monster.stuntime = monster.stuntime - dt
					else
						monster:move(monster.x_vel*dt, 0)
						local x, y = monster:center()
						monster.obj.x = x - monster.obj.width/2
						monster.obj.y = y - monster.obj.height/2

						local tile = map("trigger") and map("trigger")(math.floor(x/32), math.floor(y/32))
						if tile then--and tile.properties.ai then
							monster.x_vel = -monster.x_vel
						end

						if monster.obj.name ~= "boss1" then
							monster.anim_state = math.fmod(monster.anim_state + 3*dt, 3)
							monster.obj.gid = 695 + math.floor(monster.anim_state)
							if monster.x_vel < 0 then monster.obj.gid = monster.obj.gid + 3 end
						end
					end
				else
					monster.respawn = monster.respawn - dt
					if monster.respawn <= 0 then
						collider:setSolid(monster)
						monster.obj.visible = true
					end
				end
			end

			for i, projectile in ipairs(projectiles) do
				projectile.lifetime = projectile.lifetime - dt
				if projectile.lifetime <= 0 then
					collider:remove(projectile)
					table.remove(projectiles, i)
				else
					projectile:move(projectile.x_vel*dt, projectile.y_vel*dt)
				end
			end

			-- update the collision detection
			collider:update(dt)
		end

		-- update walls
		local walls = map("walls")
	        if walls then
                        for _, wall in pairs(walls.objects) do
                                if wall.lifetime > 0 then
                                        wall.lifetime = wall.lifetime - dt
                                        if wall.lifetime <= 0 then
                                                if wall.ghosted then
                                                        collider:setSolid(wall.ctile)
                                                        wall.visible = true
                                                else
                                                        collider:setGhost(wall.ctile)
                                                        wall.visible = false
                                                end
                                                wall.ghosted = not wall.ghosted
                                        end
                                end
                        end
                end

		--move camera
		local cam_x, cam_y = cam.pos.x, cam.pos.y
		local hero_x, hero_y = hero:center()
		local move_cam_x = closer(hero_x - cam_x, 100)
		local move_cam_y = closer(hero_y - cam_y, 100)
		cam:move(move_cam_x, move_cam_y)
	end

	--cam:move(20*dt, 0)
	if hero.dead then
		cam:rotate(0.1*dt)
	end

	--sounds.music:play()

	if current_popup.lifetime > 0 then
		current_popup.lifetime = current_popup.lifetime - dt
		if current_popup.lifetime <= 0 then
			callb = current_popup.callback
			current_popup.callback = nil
			current_popup.img = nil
			if callb then callb() end
		end
	end
end

function love.draw()
	cam:attach()

	-- draw the level
	local cam_x = cam.pos.x - love.graphics.getWidth()/2
	local cam_y = cam.pos.y - love.graphics.getHeight()/2
	map:autoDrawRange(-cam_x, -cam_y, 1, 0)
	map:draw()

	-- draw the hero as a rectangle
	--hero:draw("fill")
	local x, y = hero:center()
	x = x - 64/2
	y = y - 64+24
	love.graphics.draw(hero.img, hero.quads[hero.current_fish][hero.current_anim], (hero.facing_left and x+64) or x, y, 0, (hero.facing_left and -1) or 1, 1)

	--for i, monster in ipairs(monsters) do
	--	monster:draw("line")
	--end

	for i, projectile in ipairs(projectiles) do
		local x, y = projectile:center()
		--projectile:draw("line")
		if projectile.type == "projectile" then
			x, y = x-14, y-14
			love.graphics.draw(projectile.img, projectile.quads[2][math.floor(4*projectile.lifetime/2.5)+1], x, y, 0, 2, 2)
		elseif projectile.type == "projectile2" then
			x, y = x-8, y-9
			if projectile.x_vel >= 0 then
				love.graphics.draw(projectile.img, projectile.quads[2][math.min(4, math.floor(3*projectile.lifetime/0.4)+2)], x, y, 0, 1, 1)
			else
				love.graphics.draw(projectile.img, projectile.quads[1][math.min(4, math.floor(3*projectile.lifetime/0.4)+2)], x+8, y, 0, -1, 1)
			end
		end
	end

	cam:detach()

	for i = 1, hero.health do
		love.graphics.draw(heart, 32*i, 20)
	end
	--love.graphics.print("Health: " .. hero.health, 10, 10)

	if current_popup.lifetime > 0 then
		local img = current_popup.img
		if img then
			local imgw, imgh = img:getDimensions()
			love.graphics.draw(img, love.graphics.getWidth()/2 - imgw/2, current_popup.y)
		end
	end
end

local collision_handlers = {}

function on_collide(dt, shape_a, shape_b, mtv_x, mtv_y)

	if shape_a == hero then
		collision_handlers[shape_b.type](dt, shape_a, shape_b, mtv_x, mtv_y)
	elseif shape_b == hero then
		collision_handlers[shape_a.type](dt, shape_b, shape_a, mtv_x, mtv_y)
	end

	local projectile = (shape_a.type == "projectile" and shape_a) or (shape_b.type == "projectile" and shape_b)
	local target = (shape_a.type == "target" and shape_a) or (shape_b.type == "target" and shape_b)
	local ground = (shape_a.type == "ground" and shape_a) or (shape_b.type == "ground" and shape_b)
	local monster = (shape_a.type == "monster" and shape_a) or (shape_b.type == "monster" and shape_b)
	if projectile and (target or ground or monster) then
		projectile.lifetime = 0
		sounds.hit:play();

		if target then
			print("target hit")
			local name = target.obj.properties.target
			for _, wall in pairs(map("walls").objects) do
				if wall.name == name then
					if wall.lifetime <= 0 then
						if wall.ghosted then
							collider:setSolid(wall.ctile)
							wall.visible = true
						else
							collider:setGhost(wall.ctile)
							wall.visible = false
						end
						wall.ghosted = not wall.ghosted
					end
					if wall.properties.time then wall.lifetime = wall.properties.time end
				end
			end
		elseif monster then
			monster.stuntime = 1
		end
	end
end

function collision_handlers.ground(dt, entity, collidee, mtv_x, mtv_y)
	entity:move(mtv_x*1.01, 0)
	entity:move(0, mtv_y)

	if math.abs(mtv_x) > 0 then
		entity.x_vel = 0
		if hero.stabfly then
			hero.stabfly = false
			hero.in_control = true
			sounds.stabstop:play()
		end
	else
		if mtv_y < 0 then
			if hero.jumping then
				hero.current_anim = anims.idle1
				print("stopjump" .. mtv_y)
				hero.jumping = false
			end
			if hero.stabfly then
				hero.stabfly = false
				hero.in_control = true
				sounds.stabstop:play()
			end
		end
		if math.sign(mtv_y) == math.sign(entity.y_vel) then
			entity.y_vel = 0
		end
	end
end

function collision_handlers.spikes(dt, entity, collidee, mtv_x, mtv_y)
	hero.in_control = false
	hero.dead = true
	hero.health = -999
	hero.x_vel = -400
	hero.y_vel = 100

	sounds.death:play()

	current_popup.img = popups.death
	current_popup.lifetime = 60
	current_popup.y = 400
end

function collision_handlers.monster(dt, entity, collidee, mtv_x, mtv_y)
	if hero.dead then return end

	print("ouch")

	entity:move(mtv_x*1.01, 0)
	entity:move(0, mtv_y)

	local vel = -200
	if mtv_x > 0 then vel = -vel end
	hero.x_vel = vel
	hero.y_vel = 250
	hero.stabfly = true
	hero.in_control = false
	hero.action = false

	hero.health = hero.health - 1
	if hero.health <= 0 then
		hero.dead = true
		hero.stabfly = false --don't regain control
		sounds.death:play()

		current_popup.img = popups.death
		current_popup.lifetime = 60
		current_popup.y = 400
	end
end

function collision_handlers.pickup(dt, entity, collidee, mtv_x, mtv_y)
	local pickup = collidee.obj
	print(pickup.name)

	if not pickup.properties.pond then
		collider:remove(collidee)

		sounds.fanfare:play()
		local hx, hy = hero:center()
		pickup.x = hx-6
		if hero.facing_left then pickup.x = hx-38 end
		pickup.y = hy-16-38
		hero.in_control = false
		pause = true
		hero.x_vel = 0
		hero.action = false
		hero.current_fish = 6
		hero.current_anim = 2
		current_popup.lifetime = 2
		local hx, hy = hero:center()
		local camx, camy = cam.pos.x, cam.pos.y
		if hy > camy then
			current_popup.y = 50
		else
			current_popup.y = 320
		end
		if popups[fishes[pickup.name]] then current_popup.img = popups[fishes[pickup.name]] end
		current_popup.callback = function()
			pickup.visible = false
			hero.in_control = true
			pause = false

			if fishes[pickup.name] then
				hero.fish_inventory[fishes[pickup.name]] = true
				change_fish(fishes[pickup.name])
			end
		end
	else
		print("picking up "..pickup.properties.pond)
		collider:setGhost(collidee)
		pickup.visible = false
		hero.fish_inventory[pickup.properties.pond] = true
		change_fish(pickup.properties.pond)
		pickup.properties.pond = ""

		sounds.fanfare:play()
	end
end

function collision_handlers.sign(dt, entity, collidee, mtv_x, mtv_y)
	if hero.current_fish == fishes.none then return end
	if hero.action then return end

	local target = collidee.obj.properties.target
	for _,obj in pairs(map("fisk").objects) do
		if obj.name == target then
			if obj.properties.pond == "" then
				print("leaving "..hero.current_fish)

				obj.visible = true
				collider:setSolid(obj.ctile)
				obj.properties.pond = hero.current_fish
				obj.gid = object_offset + hero.current_fish
				if obj.gid == object_offset+5 then obj.gid = object_offset+8 end
				print("gid: " .. obj.gid)

				hero.fish_inventory[hero.current_fish] = false
				change_fish(fishes.none)

				sounds.fanfare2:play()
			end
		end
	end
end

function collision_handlers.snake(dt, entity, collidee, mtv_x, mtv_y) -- solid snake
	local ready = true
	for i = fishes.jumpfish, fishes.swordfish do
		if hero.fish_inventory[i] then ready = false end
	end

	if ready then
		if current_popup.lifetime <= 0 then
			pause = true
			current_popup.lifetime = 60
			current_popup.img = popups.success
			current_popup.y = love.graphics.getHeight()/2 - current_popup.img:getHeight()/2
		end
	else
		if current_popup.lifetime <= 0 then
			current_popup.lifetime = 5
			current_popup.img = popups.deliver
			--current_popup.y = love.graphics.getHeight()/2 - current_popup.img/2
			current_popup.y = 500
			local vel = -200
			if mtv_x > 0 then vel = -vel end
			hero.x_vel = vel
			hero.y_vel = 250
			hero.stabfly = true
			hero.in_control = false
			hero.action = false
		end
	end
end

function collision_handlers.projectile(dt, entity, collidee, mtv_x, mtv_y)
	-- not crash
end
function collision_handlers.projectile2(dt, entity, collidee, mtv_x, mtv_y)
	-- not crash
end
function collision_handlers.target(dt, entity, collidee, mtv_x, mtv_y)
	-- not crash
end

function setupHero(x, y)

	hero = collider:addRectangle(x,y,20,48)
	hero.speed = 150
	hero.accel = 600
	hero.img = love.graphics.newImage("img/binge.png")

	hero.x_vel = 0
	hero.y_vel = 0
	hero.facing_left = false

	hero.jump_vel = 350
	hero.in_control = true
	hero.dead = false
	hero.health = 3

	hero.quads = {}
	for fish_index = 1,6 do
		local fish_anims = {}
		for anim_index = 1,9 do
			fish_anims[anim_index] = love.graphics.newQuad((anim_index-1)*64, (fish_index-1)*64, 64, 64, hero.img:getDimensions())
		end
		hero.quads[fish_index] = fish_anims
	end

	hero.current_fish = 1
	hero.current_anim = anims.idle1
	hero.walk_state = 0
	hero.idle_state = 0
	hero.action_state = 0

	hero.fish_inventory = {}
	hero.fish_inventory[fishes.none] = true

	hero.jumping = false
	hero.action = false
	hero.stabfly = false
end

function animate_state(state, change, start_anim, end_anim)
	state = math.fmod(state + change, end_anim-start_anim+1)
	hero.current_anim = math.floor(start_anim + state)
	return state
end

function jump(speed)
	if not hero.jumping then
		if hero.y_vel == 0 then
			hero.y_vel = speed
			print("jump!")
		end
		hero.jumping = true
		hero.current_anim = anims.jump
	end
end

--Joystick deadzone
function deadzone(axis, deadzone)
	if math.abs(axis) < (deadzone or 0.15) then
		return 0
	else
		return axis
	end
end


function love.gamepadpressed(joystick, button)
	-- Handles the buttons
	-- Values for buttons: A, B, X, Y  https://love2d.org/wiki/GamepadButton

	if button == 'a' and not hero.action then
		jump(hero.jump_vel)
	end

	if button == 'x' then
		change_fish(fishes.swordfish)
		local fun = fish_actions.pressed[hero.current_fish]
		if fun then fun() end
	end

	if button == 'y' then
		change_fish(fishes.jumpfish)
		local fun = fish_actions.pressed[hero.current_fish]
		if fun then fun() end
	end

	if button == 'b' then
		change_fish(fishes.bubblefish)
		local fun = fish_actions.pressed[hero.current_fish]
		if fun then fun() end
	end

	if button == 'rightshoulder' then
		change_fish(fishes.glidefish)
		local fun = fish_actions.pressed[hero.current_fish]
		if fun then fun() end
	end

	if button == 'leftshoulder' then
		change_fish(fishes.none)
		local fun = fish_actions.pressed[hero.current_fish]
		if fun then fun() end
	end

	if button == 'start' then
		current_popup.lifetime = 0.1
	end

	if button == 'back' then
		love.event.push('quit')
	end
end


function love.gamepadreleased(joystick, button)
	if hero.in_control and button == 'rightshoulder' then
		local fun = fish_actions.released[hero.current_fish]
		if fun then fun() end
	end
end


function handleInput(dt)
	----------------------------------
	--		Joystick function 		--
	----------------------------------



	if joystick then

		hero.x_vel = hero.x_vel + deadzone(joystick:getAxis(1))*dt*hero.accel
		if hero.x_vel < -hero.speed then hero.x_vel = -hero.speed end
		if hero.x_vel > hero.speed then hero.x_vel = hero.speed end

		if not hero.jumping and not hero.action then
			hero.walk_state = animate_state(hero.walk_state, 4*dt, anims.walk1, anims.walk4)
		end

		if deadzone(joystick:getAxis(1)) < 0 then
			hero.facing_left = true
		elseif deadzone(joystick:getAxis(1)) > 0 then
			hero.facing_left = false
		end
	end




	if love.keyboard.isDown("left") then
		hero.x_vel = hero.x_vel - dt*hero.accel
		if hero.x_vel < -hero.speed then hero.x_vel = -hero.speed end

		if not hero.jumping and not hero.action then
			hero.walk_state = animate_state(hero.walk_state, 4*dt, anims.walk1, anims.walk4)
		end

		hero.facing_left = true
	elseif love.keyboard.isDown("right") then
		hero.x_vel = hero.x_vel + dt*hero.accel
		if hero.x_vel > hero.speed then hero.x_vel = hero.speed end

		if not hero.jumping and not hero.action then
			hero.walk_state = animate_state(hero.walk_state, 4*dt, anims.walk1, anims.walk4)
		end

		hero.facing_left = false
	elseif not joystick or deadzone(joystick:getAxis(1)) == 0 then
		hero.x_vel = hero.x_vel - (hero.x_vel*10*dt)
		if math.abs(hero.x_vel) < 10 then hero.x_vel = 0 end

		if not hero.jumping and not hero.action then
			hero.idle_state = animate_state(hero.idle_state, 1*dt, anims.idle1, anims.idle2)
		end
	end

	if love.keyboard.isDown("z") or hero.action then
		local fun = fish_actions.held[hero.current_fish]
		if fun then fun(dt) end
	end

	if (love.keyboard.isDown("up") or love.keyboard.isDown("space")) and not hero.action then
		jump(hero.jump_vel)
	end
end

function updateHero(dt)

	-- apply a downward force to the hero (=gravity)
	hero.y_vel = hero.y_vel - 60*9.82*dt
	if hero.y_vel < -400 then hero.y_vel = -400 end
	if hero.y_vel < -50 then
		jump(0)
	end

	hero:move(dt*hero.x_vel,0)
	hero:move(0,-dt*hero.y_vel)
end

function findSolidTiles(map)

	local collidable_tiles = {}

	for x, y, tile in map("ground"):iterate() do
		if tile.properties.solid then
			local padding = 0
			if tile.properties.solid == 'spikes' then
				padding = 5
			end
			local ctile = collider:addRectangle(x*32+padding,y*32+padding,32-padding*2,32-padding*2)
			ctile.type = tile.properties.solid
			collider:addToGroup("tiles", ctile)
			collider:setPassive(ctile)
			table.insert(collidable_tiles, ctile)
		end
	end

	return collidable_tiles
end
